PP.mailSceneGroup = function()
    local SV = PP.savedVars.ListStyle

    --MAIL_INBOX--MAIL_INBOX_SCENE----------------------------------------------------------------------
    local navigationTree = MAIL_INBOX.navigationTree
    local navigationContainer = navigationTree.scrollControl

    MAIL_INBOX_SCENE:RemoveFragment(FRAME_PLAYER_FRAGMENT)
    MAIL_INBOX_SCENE:RemoveFragment(RIGHT_BG_FRAGMENT)
    MAIL_INBOX_SCENE:RemoveFragment(TITLE_FRAGMENT)
    MAIL_INBOX_SCENE:RemoveFragment(MAIL_TITLE_FRAGMENT)

    PP:CreateBackground(ZO_MailInbox, nil, nil, nil, -10, -10, nil, nil, nil, 0, 10)
    PP.Anchor(ZO_MailInbox, TOPRIGHT, GuiRoot, TOPRIGHT, 0, 120, true, BOTTOMRIGHT, GuiRoot, BOTTOMRIGHT, 0, -70)
    PP.Anchor(navigationContainer, TOPLEFT, ZO_MailInbox, TOPLEFT, 0, 90, true, BOTTOMLEFT, ZO_MailInbox, BOTTOMLEFT, 0, 0)
    PP.ScrollBar(navigationContainer, 180, 180, 180, 0.7, 20, 20, 20, 0.7, false)
    ZO_Scroll_SetMaxFadeDistance(navigationContainer, SV.list_fade_distance)

    local function HighlightControl(control, animateInstantly)
        control.backdrop:SetEdgeColor(unpack(SV.list_skin_edge_sel_col))
    end

    local function UnhighlightControl(control, animateInstantly)
        control.backdrop:SetEdgeColor(unpack(SV.list_skin_edge_col))
    end

    local function TreeHeaderSetup(node, control, headerData, open, userRequested)
        control:SimpleArrowSetup(headerData.text, open)
        ZO_IconHeader_Setup(control, open, true, true, function(c) c:SetDimensions(ZO_MAIL_INBOX_KEYBOARD_NODE_WIDTH, ZO_MAIL_INBOX_KEYBOARD_NODE_HEIGHT) end)
        control.statusIcon = control.statusIcon or control:GetNamedChild("StatusIcon")
        control.statusIcon:ClearIcons()
        if NonContiguousCount(headerData.unreadData) > 0 then control.statusIcon:AddIcon(ZO_KEYBOARD_NEW_ICON) end
        if headerData.hasExpiringChildren then control.statusIcon:AddIcon(ICON_MAIL_EXPIRING) end
        if headerData.isFull then control.statusIcon:AddIcon(ICON_CATEGORY_FULL) end
        control.statusIcon:Show()
        if userRequested then
            if open then navigationTree:SelectFirstChild(node) else navigationTree:ClearSelectedNode() end
        end
    end

    local function MailEntrySetup(node, control, mailData, open)
        -- Ensure control elements are initialized
        control.subjectLabel = control.subjectLabel or control:GetNamedChild("SubjectLabel")
        control.iconTexture = control.iconTexture or control:GetNamedChild("IconTexture")
        control.expirationLabel = control.expirationLabel or control:GetNamedChild("ExpirationLabel")

        -- Check if the elements are properly initialized
        if not control.subjectLabel or not control.iconTexture or not control.expirationLabel then
            return
        end

        control.subjectLabel:SetText(mailData:GetFormattedSubject())
        local subjectColor = ZO_ColorDef:New(0.6, 0.6, 0.6)
        control.subjectLabel:SetColor(subjectColor:UnpackRGB())
        control.iconTexture:ClearIcons()
        if mailData.unread then control.iconTexture:AddIcon(ZO_KEYBOARD_NEW_ICON) end
        if mailData.firstItemIcon ~= ZO_NO_TEXTURE_FILE then control.iconTexture:AddIcon(mailData.firstItemIcon) end
        if mailData.fromSystem and mailData.category == MAIL_CATEGORY_INFO_ONLY_SYSTEM_MAIL then control.iconTexture:AddIcon("EsoUI/Art/Mail/mail_systemIcon.dds") end
        if mailData.fromCS then control.iconTexture:AddIcon("EsoUI/Art/Mail/mail_CSIcon.dds") end
        if mailData:IsExpirationImminent() then
            control.iconTexture:AddIcon(ICON_MAIL_EXPIRING)
            control.expirationLabel:SetText(zo_strformat(SI_MAIL_INBOX_EXPIRES_TEXT, mailData:GetExpiresText()))
            control.expirationLabel:SetHidden(false)
        else
            control.expirationLabel:SetHidden(true)
        end
        control.iconTexture:Show()
    end

    local function MailEntryOnSelected(control, mailData, selected, reselectingDuringRebuild)
        if selected then
            control:HighlightControl()
            if not reselectingDuringRebuild then MAIL_INBOX:RequestReadMessage(mailData.mailId) end
        elseif not control.isMouseOverTarget then
            control:UnhighlightControl()
            MAIL_INBOX:EndRead()
        end
    end

    navigationTree:AddTemplate("ZO_MailInboxHeader", TreeHeaderSetup, nil, function(leftData, rightData) return leftData.text == rightData.text end, TREE_CHILD_INDENT, 0)
    navigationTree:AddTemplate("ZO_MailInboxRow", MailEntrySetup, MailEntryOnSelected, function(leftData, rightData) return AreId64sEqual(leftData.mailId, rightData.mailId) end)
    navigationTree:AddTemplate("ZO_MailInboxEmptyRow", function(node, control, nodeData, open) control.textLabel:SetText(nodeData.text) end, nil)

    --MAIL_SEND_SCENE------------------------------------------------------------------------
    MAIL_SEND_SCENE:RemoveFragment(FRAME_PLAYER_FRAGMENT)
    MAIL_SEND_SCENE:RemoveFragment(RIGHT_BG_FRAGMENT)
    MAIL_SEND_SCENE:RemoveFragment(TITLE_FRAGMENT)
    MAIL_SEND_SCENE:RemoveFragment(MAIL_TITLE_FRAGMENT)

    PP:CreateBackground(ZO_MailSend, nil, nil, nil, -10, -10, nil, nil, nil, 0, 6)
    PP:HideBackgroundForScene(MAIL_SEND_SCENE, ZO_PlayerInventory.PP_BG)
    PP.Anchor(ZO_MailSend, TOPRIGHT, GuiRoot, TOPRIGHT, 0, 120, true, BOTTOMRIGHT, GuiRoot, BOTTOMRIGHT, 0, -90)
end